package fr.llibre.ml2;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;

public class MyTest {	

	/** 
	 *   Lecture du tableau des étoiles nommées les plus brillantes.
	 *   La lecture est effectuée dans un stream, ce qui permet de lire
	 *   dans un fichier ressource.   
	 * @param isr  : InputStreamReader du fichier des étoiles à lire
	 * @return     : le tableau des étoiles lues.
	 *   Version obsolète avec le bsNumber dans index
	 */
	public static Star[] lect139Stars(InputStreamReader isr)
	{
		int separ = '|';
		Star tabStars[] = new Star[145];
		int nbs = 0;
		try {
			BufferedReader in = new BufferedReader(isr);
			while(true)
			{
				Star e = new Star();				
				String str = in.readLine();
				if (str == null) break; // break si fin de fichier
				int n1 = 0, n2 = str.indexOf(separ);
				e.nom = str.substring(n1, n2); 
				n1 = n2+1; n2 = str.indexOf(separ, n1);
				e.id = str.substring(n1, n2);
				String constel = str.substring(n2-3, n2); // 3 lettres prises dans la designation Bayer
				e.ixCons = Astro.cons3toIndex(constel);
				n1 = n2+1; n2 = str.indexOf(separ, n1);
				//e.constel =  str.substring(n1, n2);
				n1 = n2+1; n2 = str.indexOf(separ, n1);
				e.mag = Double.parseDouble(str.substring(n1, n2));
				n1 = n2+1; n2 = str.indexOf(separ, n1);
				e.tipe = str.substring(n1, n2);
				n1 = n2+1; n2 = str.indexOf(separ, n1);				
				e.index = Integer.parseInt(str.substring(n1, n2)); // dans le BSCat.
				n1 = n2+1; n2 = str.indexOf(separ, n1);				
				e.ad0_hd = Double.parseDouble(str.substring(n1, n2));
				n1 = n2+1;							
				e.de0_dd = Double.parseDouble(str.substring(n1));
				e.flCoher = false;
				tabStars[nbs] = e;
				nbs += 1;
			}	
			in.close();
		}
		catch(Exception exc)
		{
			System.out.println("Erreur lecture fichier 139Stars" + exc.getMessage());
			exc.printStackTrace();
		}
		Star bigKnown[] = new Star[nbs];
		for (int i=0; i <nbs; i++)bigKnown[i] = tabStars[i];
		return bigKnown;
	}


	/**
	 *   Lecture du tableau des étoiles nommées les plus brillantes et synchronisation
	 *   avec les étoiles du Bright Star Catalog
	 *   La lecture est effectuée dans un stream, ce qui permet de lire
	 *   dans un fichier ressource.   
	 * @param isr  : InputStreamReader du fichier des étoiles à lire
	 * @param brightS  : Tabeau brighStar pour synchronyser l'étoile avec le tableau ou null 
	 * @return     : le tableau des étoiles lues.
	 */
	public static Star[] lect139Stars(InputStreamReader isr, Etoile brightS[])
	{
		int separ = '|';
		Star tabStars[] = new Star[145];
		int nbs = 0, bsNumber, nok = 0;
		try {
			BufferedReader in = new BufferedReader(isr);
			while(true)
			{
				Star e = new Star();				
				String str = in.readLine();
				if (str == null) break; // break si fin de fichier
				int n1 = 0, n2 = str.indexOf(separ);
				e.nom = str.substring(n1, n2); 
				n1 = n2+1; n2 = str.indexOf(separ, n1);
				e.id = str.substring(n1, n2);
				String constel = str.substring(n2-3, n2); // 3 lettres prises dans la designation Bayer
				e.ixCons = Astro.cons3toIndex(constel);
				n1 = n2+1; n2 = str.indexOf(separ, n1);
				//				e.constel =  str.substring(n1, n2);
				n1 = n2+1; n2 = str.indexOf(separ, n1);
				e.mag = Double.parseDouble(str.substring(n1, n2));
				n1 = n2+1; n2 = str.indexOf(separ, n1);
				e.tipe = str.substring(n1, n2);
				n1 = n2+1; n2 = str.indexOf(separ, n1);				
				bsNumber = Integer.parseInt(str.substring(n1, n2));
				n1 = n2+1; n2 = str.indexOf(separ, n1);				
				e.ad0_hd = Double.parseDouble(str.substring(n1, n2));
				n1 = n2+1;							
				e.de0_dd = Double.parseDouble(str.substring(n1));
				e.flCoher = false;
				if (brightS != null)
				{
					Etoile s = brightS[bsNumber];
					if(s.flCoher)
					{
						double cosDec = Math.cos(Math.toRadians(s.de0_dd));
						double da_dd = (e.ad0_hd - s.ad0_hd)*15.; 
						da_dd = ((da_dd + 180.)%360.-180.);
						da_dd *= cosDec;
						double dd_dd = (e.de0_dd - s.de0_dd + 180.)%360. - 180.;
						double dist_dm = Math.sqrt(da_dd*da_dd+dd_dd*dd_dd)*60.;
						// Distance inférieure à 1 minute de degré ?
						if (dist_dm <= 1) 
						{
							e.flCoher = true;
							// Concordance trouvée, on prend les valeurs du Bright Star Catalogue
							e.ad0_hd = s.ad0_hd;
							e.de0_dd = s.de0_dd;
							e.vA_sda = s.vA_sda;
							e.vD_sda = s.vD_sda;
							//							e.vitMod_sda = s.vitMod_sda;
							e.mag = s.mag;
							e.tipe = s.tipe;
							e.id = s.id;
						}
						else nok++;
					}
				}
				e.index = nbs;
				tabStars[nbs] = e;
				nbs += 1;
			}	
			in.close();
			if (nok > 0) 
				System.out.println(nok + "étoiles sans concordance dans le fichier de 139 étoiles brillantes !");

		}
		catch(Exception exc)
		{
			System.out.println("Erreur lecture fichier 139Stars" + exc.getMessage());
			exc.printStackTrace();
		}
		Star bigKnown[] = new Star[nbs];
		for (int i=0; i <nbs; i++)bigKnown[i] = tabStars[i];
		return bigKnown;
	}


	/**
	 * Synchronisation des tableaux d'étoiles 139 et 9110
	 * Renvoi le nb de mauvaises correspondances (> à 1 minute de degré)
	 * Si pas d'erreur, ce nombre doit ^tre égal à 0.
	 * 
	 * @param bigK    : tableau des étoiles nommées
	 * @param brightS : tableau Bright Star Catalogue
	 * @return        : nombre de mauvaises correspondances (doit être égal à 0)
	 * Version obsolète ; appeler directement lect139Stars(isr, brightS)
	 * 
	 */
	public static int synchroStars(Star bigK[], Etoile brightS[])
	{
		int nok = 0;
		for (int i = 0; i < bigK.length; i++)
		{
			Star e = bigK[i];
			int n = e.index; // recupère le bsNumber
			Etoile s = brightS[n];
			if(!s.flCoher)continue ;
			double cosDec = Math.cos(Math.toRadians(s.de0_dd));
			double da_dd = (e.ad0_hd - s.ad0_hd)*15.; 
			da_dd = ((da_dd + 180.)%360.-180.);
			da_dd *= cosDec;
			double dd_dd = (e.de0_dd - s.de0_dd + 180.)%360. - 180.;
			double dist_dm = Math.sqrt(da_dd*da_dd+dd_dd*dd_dd)*60.;
			// Distance supérieure à 1 minute de degré ?
			if (dist_dm > 1) {nok++; e.flCoher = false; continue;}
			// Concordance trouvée, on prend les valeurs du Bright Star Catalogue
			e.flCoher = true;
			e.ad0_hd = s.ad0_hd;
			e.de0_dd = s.de0_dd;
			e.vA_sda = s.vA_sda;
			e.vD_sda = s.vD_sda;
			//			e.vitMod_sda = s.vitMod_sda;
			e.mag = s.mag;
			e.tipe = s.tipe;
			e.id = s.id;			
		}
		return nok;		
	}


	/**
	 *  Lecture des 2 fichiers d'étoiles (nommées et Bright Star Catalog)
	 *  supposés être situés dans le répertoire courant du projet Java
	 *  Méthode utilisée uniquement en mode Java pur.
	 *  
	 * @param bigKnown    : reférence pour le retour du tableau des étoiles nommées
	 * @param brightStars : reférence pour le retour du tableau Bright Star Catalog
	 * @return            : true si lecture ok.
	 * Version obsolete
	 */	
	public static Star [] load139Stars()
	{
		Star [] bigKnown = null;
		FileInputStream fis1;
		try {
			fis1 = new FileInputStream(new File("stars139.txt"));
			InputStreamReader isr1 = new InputStreamReader(fis1, "UTF-8");
			bigKnown = lect139Stars(isr1);		
			fis1.close();
		}		
		catch(Exception exc)
		{
			System.out.println(exc.getMessage());
			exc.printStackTrace();
		}
		return bigKnown;
	}

	/**
	 *  Lecture des 2 fichiers d'étoiles (nommées et Bright Star Catalog)
	 *  supposés être situés dans le répertoire courant du projet Java
	 *  Méthode utilisée uniquement en mode Java pur.
	 *  
	 * @param bigKnown    : reférence pour le retour du tableau des étoiles nommées
	 * @param brightStars : reférence pour le retour du tableau Bright Star Catalog
	 * @return            : true si lecture ok.
	 * Version obsolete utiliser load139Stars(brightS);
	 */	
	public static Star [] load139Stars(Etoile brightS[])
	{
		Star [] bigKnown = null;
		FileInputStream fis1;
		try {
			fis1 = new FileInputStream(new File("stars139.txt"));
			InputStreamReader isr1 = new InputStreamReader(fis1, "UTF-8");
			bigKnown = lect139Stars(isr1, brightS);		
			fis1.close();
		}		
		catch(Exception exc)
		{
			System.out.println(exc.getMessage());
			exc.printStackTrace();
		}
		return bigKnown;
	}



	/**
	 *   Lecture du tableau des 9110 étoiles du BrightStar Catalog
	 *   La lecture est effectuée dans un stream, ce qui permet de lire
	 *   dans un fichier ressource.
	 *   
	 * @param isr  : InputStreamReader du fichier des étoiles à lire
	 * @return     : le tableau des étoiles lues.
	 */
	public static Etoile[] lect9110Stars(InputStreamReader isr)
	{
		final int MAXSTARS = 9110;
		Etoile tabStars[] = new Etoile[MAXSTARS+1];
		int nbs = 0;
		tabStars[0] = new Etoile(); tabStars[0].flCoher = false;// On commence à 1
		try {
			BufferedReader in = new BufferedReader(isr);
			byte []line;
			while(nbs < MAXSTARS)
			{
				nbs += 1;
				Etoile e = new Etoile();
				tabStars[nbs] = e;
				try {
					String str = in.readLine();
					if (str == null) break; // break si fin de fichier
					line = str.getBytes();				
					if (line.length < 107) continue; // break si lignes incompletes
					e.bsNumber = Integer.parseInt(new String(line,0,4).trim());
					e.id = new String(line,4,10).trim();
					if (e.id.length() == 0)
						e.id = new String(line,51,9).trim();
					// Lignes bizarres, on ne teste que l'ascension droite)
					if(line[76]==' ' || line[78]==' ' || line[80]==' '){ e.flCoher = false; continue;}
					int HH = Integer.parseInt(new String(line,75,2));
					int MM = Integer.parseInt(new String(line,77,2));
					double SS = Double.parseDouble(new String(line,79,4));
					e.ad0_hd = HH + MM/60. + SS/3600.;
					int DD = Integer.parseInt(new String(line,84,2));
					int MD = Integer.parseInt(new String(line,86,2));
					int SD = Integer.parseInt(new String(line,88,2));
					e.de0_dd = DD + MD/60. + SD/3600.;
					if (line[83]=='-') e.de0_dd = -e.de0_dd;
					e.tipe = new String(line,127,20).trim();
					double cosDec = Math.cos(Math.toRadians(e.de0_dd));
					double vX_sda = Double.parseDouble(new String(line,148,6));
					e.vD_sda = Double.parseDouble(new String(line,154,6));
					e.vA_sda = vX_sda/cosDec;
					//					e.vitMod_sda = Math.sqrt(vX_sda*vX_sda + e.vD_sda*e.vD_sda);
					e.mag = Double.parseDouble(new String(line,102,5));
					e.flCoher = true;
				}
				catch(Exception exc)
				{
					e.flCoher = false;
					System.out.println("Erreur lecture fichier 9110Stars ligne " + nbs + "; " + exc.getMessage());
					//	Toast.makeText(this, "Erreur lecture fichier " + exc.getMessage(), Toast.LENGTH_LONG).show(); return;
				}
			}	
			in.close();
		}
		catch(Exception exc)
		{
			System.out.println("Erreur lecture fichier 9110Stars ligne " + nbs);
			System.out.println(exc.getMessage());
			exc.printStackTrace();
			// Toast.makeText(this, "Erreur lecture fichier " + exc.getMessage(), Toast.LENGTH_LONG).show(); return;
		}
		return tabStars;
	}



	/**
	 *  Lecture du Bright Star Catalog
	 *  supposés être situés dans le répertoire courant du projet Java
	 *  Méthode utilisée uniquement en mode Java pur.
	 *  NON UTILISEE SOUS ANDROID
	 *  
	 * @return  : Le tableau lu, null si erreur
	 */

	public static Etoile[] loadBrightStarCatalog()
	{
		Etoile[] brightStars = null;
		FileInputStream fis2;
		try {
			fis2 = new FileInputStream(new File("stars9110.txt"));
			InputStreamReader isr2 = new InputStreamReader(fis2, "UTF-8");
			brightStars = lect9110Stars(isr2);	
			fis2.close();
		}		
		catch(Exception exc)
		{
			System.out.println(exc.getMessage());
			exc.printStackTrace();
		}
		return brightStars;
	}



	public static void testLectStars() {	

		// Test fichier 139 et 9110	
		Etoile brightStar[] = loadBrightStarCatalog();
		if (brightStar == null)
		{
			System.out.println("Erreur lecture brightStar!");
			return;
		}
		if (brightStar.length != 9111)
		{
			System.out.println("Erreur lecture brightStar! : trouvé " + brightStar.length + " étoiles");
			return;
		}			
		Star bigKnown[] = load139Stars(brightStar); 	
		if (bigKnown == null)
		{
			System.out.println("Erreur lecture bigKnown!");
			return;
		}
		if (bigKnown.length != 139)
		{
			System.out.println("Erreur lecture bigKnown! : trouvé " + bigKnown.length + " étoiles");
			return;
		}				
		String nomStar;

		nomStar = "Polaire";
//		double ju =  2431559.7;
		double ju = Utils.julNow();
		double longi_dg = 0;
		double lati_dg = 45;
		int p = Star.find(nomStar, bigKnown);
		System.out.println(nomStar + " : " );
		if (p > 0)
		{
			Star pol = bigKnown[p];
			Astro.update_eco(ju);
			System.out.println(pol.infos(ju, longi_dg, lati_dg, null, 1, true));
			double tHA_dg[] = {0.};
			double pis = Astro.polarisInScope_dg(ju, 1.5731, tHA_dg, true);
			System.out.println("Polaris in Scope : " + Utils.deg2hms(pis));
			System.out.println("Angle horaire    : " + Utils.deg2hms(tHA_dg[0]));
		}
		double tsa_dg = Astro.jul2alst_dg(ju, longi_dg, false);		
		System.out.println("\nHeure sidérale apparente : " + Utils.deg2hms(tsa_dg) + "\n");
		nomStar = "regulus";
		p = Star.find(nomStar, bigKnown);
		if (p > 0)
		{
			double result[] = new double[14];
			Star reg = bigKnown[p];
			System.out.println(reg.nom + " : ");
			System.out.println(reg.infos(ju, longi_dg, lati_dg, result, 1, true));

			ArrayList<Etoile> prox = (ArrayList<Etoile>)reg.proxy(brightStar, 5);
			int nprox = prox.size();
			System.out.println("\nListe des " + nprox + " étoiles proches de " + reg.nom);
			for (int i = 0; i < nprox; i++)
			{
				Etoile e = prox.get(i);
				System.out.println("BSname = " + e.id + ", distance = " + e.work + ", mag = " + e.mag);
			}
			// Vérification des transformations coord.equa / J2000 en coord.loca apparentes par calcul 
			// aller et retour et distance entre les deux directions
			double azi_dg = result[5];
			double sit_dg = result[7];		
			Star.distDir(bigKnown, azi_dg, sit_dg, longi_dg, lati_dg, ju);
			System.out.println("Distance de Régulus à lui même : " + reg.work + " degrés");
		}			
	}
	
	

	public static void testSatel() {	
//		double ju = Utils.dat2jul(2017, 10, 27, 1, 0, 0);
		double ju = Utils.julNow();
//		double ju = 2458045.73117419;
		String info = "Date : " + Utils.jul2str1(ju) + "\n";
		double longi_dg = 1.4442; double lati_dg = 43.6047; double alti_m = 0.;
		int sexaDec = 1;
		
		// La station ISS
		Sater iss = new Sater(TwoLE.makeNew(TwoLE.url[0], 0));
		info += "Satellite : " + iss.name + "\n";
		info +=	iss.infos(ju, longi_dg, lati_dg, alti_m, sexaDec);
		System.out.println(info);
		
		info = "\n";	

		// Le satellite ASTRA 1D
		Sater astra = new Sater(TwoLE.makeNew(TwoLE.url[7], 4));
		info += "Satellite : " + astra.name + "\n";
		info +=	astra.infos(ju, longi_dg, lati_dg, alti_m, sexaDec);
		System.out.println(info);
		
	}
	
	// Calcul des distances angulaires de la Lune et des planètes au Soleil
	// Les distances euclidiennes sont en UA.
	public static double[] distoSun(double ju)
	{
		double angles [] = new double[9];
		double deltaT =  Utils.jul2deltaT(ju);
		double je = ju + deltaT;				
		double xyzlbr[][] = new double[9][6];
	    // Position des planètes par / au Soleil dans rep. Eclip. 
		for (int nopla = 0; nopla < 8; nopla++)
//			PlanetVsop87.position(je, nopla, xyzlbr[nopla]);
			PlanetEop.position(je, nopla, xyzlbr[nopla]);
				
		// Position des planètes / Terre dans rep. Eclip. Retard vitesse lumière négligé		
		for (int nopla = 0; nopla < 8; nopla++)
		{		
			if (nopla != 2)
			{
				// Position planete / Terre
				double rho1 = 0.;
				for(int i = 0; i < 3; i++) 
				{					
					xyzlbr[nopla][i] = xyzlbr[nopla][i] - xyzlbr[2][i]; // TP = SP - ST
					rho1 += xyzlbr[nopla][i]*xyzlbr[nopla][i];
				}
				xyzlbr[nopla][5] = Math.sqrt(rho1);
			}			
		}
		// Position Soleil / Terre 
		// (Ne pas faire dans la boucle ci-avant sinon erreur pour i > 2)
		for(int i = 0; i < 3; i++) xyzlbr[2][i] = -xyzlbr[2][i]; 
		
		// Position Lune / Terre dans rep. Eclip.
		double br[] = new double[2];
		double l = Moon.je2eclip_rd(je, br);  
		double cb = Math.cos(br[0]);
		double rho =  br[1]/Astro.UA;
		xyzlbr[8][0] = rho * Math.cos(l) * cb;
		xyzlbr[8][1] = rho * Math.sin(l) * cb;
		xyzlbr[8][2] = rho * Math.sin(br[0]);
		xyzlbr[8][5] = rho;
		// calcul distances angulaires géocentriques Planètes par rapport à Gamma 
		for (int nopla = 0; nopla < 9; nopla++)
			angles[nopla] = Math.atan2(xyzlbr[nopla][1], xyzlbr[nopla][0]);
		return angles;		
	}
	
	static void testDistoSun()
	{
		double ju = Utils.julNow();
		double angles [] = distoSun(ju);
		for (int i = 0; i < 9; i++)
		{
			System.out.println("Long geo/Soleil : " + Utils.mod2(Math.toDegrees(angles[i] - angles[2]), 360));
		}
	}
	
	public static void main(String[] args) {
//	    testLectStars();
//		testSatel();
		testDistoSun();		
	}
}
