package fr.llibre.ml2;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Collections;

/**
 *  Classe Star étendant la classe étoile et comprenant en plus des paramètres
 *  quasiment constant les paramètres qui varient lentement et ceux qui 
 *  varient rapidement.
 *
 * @author llibre
 *
 */
public class Star extends Etoile {
	public int index; // Index dans un tableau pour suivi après tri
	public String nom; // Nom commun en français si possible ou autre
	public int hip; // Numero Hipparcos si présent
	public int ixCons; // Index constellation (0 à 87)
	//	public String constel; // Nom constellation (3 lettres en général)
//	public String distance;  // distance (à préciser ultérieurement)
	// Date des informations variables
	public double juCou = -1;	
	// Informations variant lentement
	public double alfa_hd;
	public double decli_dg;
	// Informations variant rapidement
	public double hourAng_hd;
	public double azi_dg;
	public double haut_dg;
	public double hautCor_dg;
	public double jtran;
	public double htran_dg;
	public double jlev;
	public double azilev_dg;
	public double jpon;
	public double azipon_dg;
	public int nbLiens;
	public int liens[];
	public boolean fInScreen;
	public float px, py; // Coordonnees dans le viseur
	/**
	 * 	Mise à jour des infos sur l'étoile et génération d'un string contenant ces infos.
	 * @param ju       : Date jour julien UT
	 * @param longi_dg : Longitude du lieu (degrés)
	 * @param lati_dg  : Latitude du lieu (degrés)
	 * @param sexaDec  : -1 : String result en décimal, 0 : neant , +1 en sexagésimal
	 * @param eco      : true => utilisation des paramètres variant lentement
	 * @return         : String contenant toutes les infos.
	 */
	public String infos(double ju, double longi_dg, double lati_dg, int sexaDec, boolean eco)
	{	
		juCou = ju;
		double [] result = new double[14];
		String mes = super.infos(ju, longi_dg, lati_dg, result, sexaDec, eco);
		alfa_hd = 15.*result[2];
		decli_dg = result[3];
		hourAng_hd = 15.*result[4];
		azi_dg = result[5];
		haut_dg = result[6];
		hautCor_dg = result[7];
		jtran = result[8];
		htran_dg = result[9];
		jlev = result[10];
		azilev_dg =	result[11];
		jpon = result[12];
		azipon_dg =	result[13];
		return mes;
	}

	/** 
	 *   Lecture du tableau des mes XXX étoiles les plus brillantes.
	 *   La lecture est effectuée dans un stream, ce qui permet de lire
	 *   dans un fichier ressource.   
	 * @param isr  : InputStreamReader du fichier des étoiles à lire
	 * @return     : le tableau des étoiles lues.
	 */
	public static Star[] loadMesStars(InputStreamReader isr, int maxstars)
	{
		int separ = '|';
		Star mesStars[] = new Star[maxstars];
		int numline = 0, nbs = 0;
		try {
			BufferedReader in = new BufferedReader(isr);
			while(nbs < maxstars)
			{
				Star e = new Star();
				String str = in.readLine();
				if (str == null) break; // break si fin de fichier
				numline++;
				if (str.startsWith("#")) continue;
				int n1 = 0, n2 = str.indexOf(separ);
				e.id = str.substring(n1, n2);
				e.ixCons = Astro.cons3toIndex(str.substring(n2-3, n2));
				n1 = n2+1; n2 = str.indexOf(separ, n1);
				if(n2 > n1+1) e.nom = str.substring(n1, n2).trim();
				else e.nom = "";
				if (e.nom.length() == 0) e.nom = "~" + e.id;
				n1 = n2+1; n2 = str.indexOf(separ, n1);
				e.bsNumber = Integer.parseInt(str.substring(n1, n2));
				n1 = n2+1; n2 = str.indexOf(separ, n1);
				e.hip = Integer.parseInt(str.substring(n1, n2));
				n1 = n2+1; n2 = str.indexOf(separ, n1);
				e.ixCons = Integer.parseInt(str.substring(n1, n2));
				n1 = n2+1; n2 = str.indexOf(separ, n1);				
				e.mag = Double.parseDouble(str.substring(n1, n2));
				n1 = n2+1; n2 = str.indexOf(separ, n1);
				e.ad0_hd = Double.parseDouble(str.substring(n1, n2));
				n1 = n2+1; n2 = str.indexOf(separ, n1);
				e.de0_dd = Double.parseDouble(str.substring(n1, n2));
				n1 = n2+1; n2 = str.indexOf(separ, n1);				
				e.vA_sda = Double.parseDouble(str.substring(n1, n2));
				n1 = n2+1; n2 = str.indexOf(separ, n1);
				e.vD_sda = Double.parseDouble(str.substring(n1,n2));
				n1 = n2+1; n2 = str.indexOf(separ, n1);
				if (n2 >= n1+1) { // A priori il y a des liens
					e.nbLiens = Integer.parseInt(str.substring(n1, n2));
					if (e.nbLiens > 0)
					{
						e.liens = new int[e.nbLiens];
						for (int ns = 0; ns < e.nbLiens; ns++)
						{
							int li ;
							n1 = n2+1;
							if (ns == e.nbLiens-1) li = Integer.parseInt(str.substring(n1));
							else
							{
								n2 = str.indexOf(separ, n1);
								li = Integer.parseInt(str.substring(n1, n2));
							}
							e.liens[ns] = li;
						}
					}
				}
				else { // Pas de liens a priori.
					e.nbLiens = Integer.parseInt(str.substring(n1));
					if (e.nbLiens != 0) {
						System.out.println("Erreur lecture fichier mes Stars ligne " + numline);
						continue;
					}
				}
				e.index = nbs;
				e.flCoher = true; // a priori
				mesStars[nbs] = e;
				nbs += 1;
			}	
			in.close();
		}
		catch(Exception exc)
		{
			System.out.println("Erreur lecture fichier mes Stars ligne " + numline + " : " + exc.getMessage());
			exc.printStackTrace();
		}
		return mesStars;
	}


	/**
	 *  Recherche par son nom d'une étoile dans le tableau des étoiles nommées
	 *  La correspondance est trouvée même avec des différence d'accents ou de casse
	 *  
	 * @param nomStar : Nom à rechercjer
	 * @param bigK    : Tableau dans lequel on recherche
	 * @return        : index de l'étoile dans le tableau ou -1 si pas trouvé.
	 */
	public static int find(String nomStar, Star bigK[])
	{
		String nom = Utils.sansAccents(nomStar);
		for (int n = 0; n <  bigK.length; n++)
		{
			String name = Utils.sansAccents(bigK[n].nom);
			if(nom.equalsIgnoreCase(name)) return n;
		}
		return -1;
	}


	public static ArrayList<Star> proxy(Star tabStars[], int nproxmax, double ad_dg, double de_dg, double magmax)
	{
		double ad_rd, de_rd;
		ad_rd = Math.toRadians(ad_dg); de_rd = Math.toRadians(de_dg);
		double ca, sa, cd, sd;
		ca = Math.cos(ad_rd); sa = Math.sin(ad_rd);
		cd = Math.cos(de_rd); sd = Math.sin(de_rd);
		double u, v, w;
		u = ca*cd; v = sa*cd; w = sd;	

		ArrayList<Proxy> prox = new ArrayList<Proxy>();
		int nprox = 0;
		double distmin = 1.e+10, dist;
		for (int i = 0; i < tabStars.length; i++)
		{
			Star e = tabStars[i];
			if (!e.flCoher || e.mag > magmax) continue;
			ad_rd = Math.toRadians(e.ad0_hd*15.); de_rd = Math.toRadians(e.de0_dd);
			ca = Math.cos(ad_rd); sa = Math.sin(ad_rd);
			cd = Math.cos(de_rd); sd = Math.sin(de_rd);
			double x, y, z;
			x = ca*cd; y = sa*cd; z = sd;	
			dist = Utils.acos(u*x+v*y+w*z);
			if (nprox < nproxmax || dist < distmin)
			{
				Proxy p = new Proxy(dist, i);
				distmin = dist;
				if (nprox < nproxmax) {nprox++; prox.add(p);}
				else
				{
					Collections.sort(prox);
					prox.remove(nproxmax-1);
					prox.add(0, p);
				}
			}
		}	
		Collections.sort(prox);
		ArrayList<Star> starProx = new ArrayList<Star>();

		for (int i = 0; i < prox.size(); i++)
		{
			Star e = tabStars[prox.get(i).n];
			e.work = prox.get(i).dist;
			starProx.add(e);
		}			
		return starProx;	
	}	


	/**
	 *      Calcul pour toutes les étoiles d'un tableau de leurs distances angulaires à une direction donnée	
	 *      Cette distance est mémorisée dans le parametre "work".
	 * @param tabStars     : Le tableau à traiter
	 * @param azimut_dg    : Azimut de la direction
	 * @param site_dg      : Site de la direction
	 * @param longitude_dg : Longitude du lieu
	 * @param latitude_dg  : Longitude du lieu
	 * @param ju	       : date julienne UT
	 */
	public static void distDir(Star tabStars[], double azimut_dg, double site_dg,
                               double longitude_dg, double latitude_dg, double ju)
	{
		// Calcul préalable des coordonnées équatoriales apparentes de la direction visée
		// Elle est corrigée de la réfraction, puis calculées en coordonnées équatoriales apparentes
		// dans le repère de la date, uis calculée en coordonnées équatoriales moyennes dans le repère J2000,
		// à la date donnée.
		// Ce calcul n'étant fait que pour la direction de visée, on n'utilise pas le mode économique
		// utilisant les paramètres astro variant lentement précalculés pour la date juIni.
		double lat_rd = Math.toRadians(latitude_dg);
		double clat = Math.cos(lat_rd);
		double slat = Math.sin(lat_rd);
		double azi_rd = Math.toRadians(azimut_dg);
		double cazi = Math.cos(azi_rd);
		double sazi = Math.sin(azi_rd);
		// Correction refraction atmosphérique
		site_dg -= Astro.refraction_dg(site_dg, false);
		double sit_rd = Math.toRadians(site_dg);
		double csit = Math.cos(sit_rd);
		double ssit = Math.sin(sit_rd);
		// Angle horaire apparent
		double H_rd = Math.atan2(sazi, slat*cazi + clat*ssit/csit);
		// Coordonnées équatoriales apparentes
		double delta_rd = Utils.asin(slat*ssit-clat*cazi*csit);
		double agst_dg = Astro.jul2agst_dg(ju, false);
		double alfa_rd = Math.toRadians(agst_dg + longitude_dg) - H_rd;
		// Remontée aux coordonnées équatoriales moyennes
		double je = ju + Utils.jul2deltaT(ju);
		// Calcul correction nutation en coordonnées écliptiques
		double dpsi_rd, dobliq_rd, tabdob[] = {0};
		dpsi_rd = Astro.nutationEcli_rd(je, true, tabdob); dobliq_rd = tabdob[0];
		// Obliquité vrai à cette date
		double obliq_rd = Math.toRadians(Astro.obliquity_dg(je)) + dobliq_rd;
		double S = (je - 2451545)/36525.;			
		double dAlpha_rd, dDelta_rd, tabDelta[] = {0};
		// Calcul correction nutation et aberration (à enlever)
		dAlpha_rd = Astro.aberrEqua_rd_std(alfa_rd, delta_rd, obliq_rd,
				Math.toRadians(Astro.sun_longi_dg(S)),
				Astro.eccentricity(S), Math.toRadians(Astro.earth_longiPerih_dg(S)), tabDelta);
		dDelta_rd = tabDelta[0];
		alfa_rd -= dAlpha_rd; delta_rd -= dDelta_rd; 
		dAlpha_rd = Astro.nutationEqua(alfa_rd, delta_rd, obliq_rd, dpsi_rd, dobliq_rd, tabDelta);
		dDelta_rd = tabDelta[0];		
		alfa_rd -= dAlpha_rd; delta_rd -= dDelta_rd; 
		// remontée aux coordonnées équatoriales dans J2000
		double []tabde = new double[1]; 
		double ra_rd = Astro.chDateCoorEqua_rd_std(alfa_rd, delta_rd, je, 2451545, tabde);
		double de_rd = tabde[0];  
		double ca, sa, cd, sd;
		ca = Math.cos(ra_rd); sa = Math.sin(ra_rd);
		cd = Math.cos(de_rd); sd = Math.sin(de_rd);
		double u, v, w;
		u = ca*cd; v = sa*cd; w = sd;	       
		double t = (ju - 2451545)/365.25;
		for (int i = 0; i < tabStars.length; i++)
		{
			Star e = tabStars[i];
			if (!e.flCoher) continue;
			ra_rd = Math.toRadians(e.ad0_hd*15.);
			de_rd = Math.toRadians(e.de0_dd);
			double vA_rda = Math.toRadians(e.vA_sda/3600.);
			double vD_rda = Math.toRadians(e.vD_sda/3600.);
			ra_rd += vA_rda*t;
			de_rd += vD_rda*t;
			ca = Math.cos(ra_rd); sa = Math.sin(ra_rd);
			cd = Math.cos(de_rd); sd = Math.sin(de_rd);
			double x, y, z;
			x = ca*cd; y = sa*cd; z = sd;	
			e.work = Utils.acos(u*x+v*y+w*z);
		}
	}
}

