package fr.llibre.ml2;

/**
 * Created by llibre on 10/12/2017.
 */

public class SysSolar {

    // Coordonnées écliptiques géocentriques des planètes, du Soleil et de la Lune dans le
    // repère écliptique moyen de la date
    // Renvoi uvwr[nopla][k] avec 0 <= nopla < 9 et 0 <= k < 6
    // nopla = 0:mercure, 1:Vénus, 2:Soleil, 3:Mars, ... , 7:Neptune, 8:Lune
    // 0:u, 1:v, 2:w : cosinus directeurs de la direction
    // 3:rho en UA sauf en mètres pour la Lune
    // Version utilisant des éphémérides simples, sans correction vitesse lumière
    public static double[][] coordEcliGeo(double ju)
    {
        double deltaT =  Utils.jul2deltaT(ju);
        double je = ju + deltaT;
        double xyzlbr[][] = new double[9][6];
        double uvwr[][] = new double[9][4];
        // Position des planètes par / au Soleil dans rep. Eclip.
        for (int nopla = 0; nopla < 8; nopla++) PlanetEop.position(je, nopla, xyzlbr[nopla]);
        // Position des planètes / Terre dans rep. Eclip. Retard vitesse lumière négligé
        // seuls xyz et r seront valides en geocentrique
        for (int nopla = 0; nopla < 8; nopla++) {
            if (nopla != 2) {
                // Position planete / Terre
                double rho1 = 0.;
                for(int i = 0; i < 3; i++) {
                    uvwr[nopla][i] = xyzlbr[nopla][i] - xyzlbr[2][i]; // TP = SP - ST
                    rho1 += uvwr[nopla][i]*uvwr[nopla][i];
                }
                rho1 =  Math.sqrt(rho1);
                uvwr[nopla][3] = rho1;
                for(int i = 0; i < 3; i++) uvwr[nopla][i] /= rho1;
            }
        }

        // Position Soleil / Terre.
        for(int i = 0; i < 3; i++) uvwr[2][i] = -xyzlbr[2][i]/xyzlbr[2][5];
        uvwr[2][3] = xyzlbr[2][5];

        // Position Lune / Terre dans rep. Eclip.
        double br[] = new double[2];
        double longi = Moon.je2eclip_rd(je, br);
        double cb = Math.cos(br[0]);
        uvwr[8][0] = Math.cos(longi) * cb;
        uvwr[8][1] = Math.sin(longi) * cb;
        uvwr[8][2] = Math.sin(br[0]);
        uvwr[8][3] = br[1];
        return uvwr;
    }

    // Coordonnées équatoriales géocentriques des planètes, du Soleil et de la Lune
    // Dans le repère équatorial vrai de la date
    // Renvoi xyzlbr[nopla][k] avec 0 <= nopla < 9 et 0 <= k < 6
    // nopla = 0:mercure, 1:Vénus, 2:Soleil, 3:Mars, ... , 7:Neptune, 8:Lune
    // 0:u, 1:v, 2:w : cosinus directeurs de la direction
    // 3:rho en UA
    // Version utilisant des éphémérides simples, sans correction vitesse lumière
    public static double[][] coordEquaGeo(double ju)
    {
        double uvwr[][] = coordEcliGeo(ju);
        for (int i = 0; i < uvwr.length; i++)
            R3.madvec(uvwr[i], Astro.m3_CvE, uvwr[i]);
        return uvwr;
    }
}
