package fr.llibre.ml2;

import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;

public final class Transit {
	/**
	 * La classe Transit ne doit pas etre instanciee.
	 */
	private Transit() {}		

	// Constantes ou données initiales
	private static final double omegaT_dgj = 360.98564736629; // vrot_terre degrés/jour


	private static double ju2equa_rd(double ju, double []pdec, boolean eco, Object o)
	{

		if (o.getClass() == Celobj.class ||
				o.getClass() == Star.class ||
				o.getClass() == Etoile.class)
			return ((Etoile) o).ju2equa_rd(ju, pdec, eco);
		else if (o.getClass() == Moon.class) return Moon.ju2equa_rd(ju, pdec, eco);
		else if(o.getClass() == Planet.class)
		{	
			Planet p = (Planet) o;
			p.ju2ecliequa_rd(ju, true, eco);
			pdec[0] = p.decli_rd;
			return p.ascDr_rd;
		}
		return 0;
	}

	// valeur type, utilisé dans le cas du Soleil : 0 : rasant, 1 civil, 2 nautique, 3 astronautique
	private static double jrefEfho_dg(double ju, Object o, double []ph0_dg, int type)
	{
		double jref = 0, h0_dg = 0;	
		if (o.getClass() == Celobj.class ||
				o.getClass() == Star.class ||
				o.getClass() == Etoile.class)
		{
			h0_dg = -0.5667; // Refraction étoile au niveau de l'horizon
			jref = Math.floor(ju + 0.5)+0.5; // minuit local de ce jour
		}
		else if (o.getClass() == Moon.class)
		{
			h0_dg = 0.125; // Refraction lune au niveau de l'horizon
			jref = ju; // temps courant
		}
		else if(o.getClass() == Planet.class)
		{	
			Planet p = (Planet) o;
			if (p.numPla == 2) // cas Soleil
			{
				switch(type)
				{
				case 1 : h0_dg = -6.; break;
				case 2 : h0_dg = -12.; break;
				case 3 : h0_dg = -13.; break;
				default : h0_dg = -0.8333; break; // Refraction soleil au niveau de l'horizon
				}
				jref = Math.floor(ju + 0.5); // midi ce jour
			}
			else // cas planète
			{
				h0_dg = -0.5667; // Refraction planete au niveau de l'horizon
				jref = Math.floor(ju + 0.5)+0.5; // minuit ce jour
			}
		}
		ph0_dg[0] = h0_dg;
		return jref;
	}

	public static boolean getResult(Object o, double ju, double longi_dg, double lati_dg,
                                    boolean eco, double []result) {
		return getResult(o, ju, longi_dg, lati_dg,eco, result, 0);
	}	
	
	/**
	 * 
	 * @param o        : Objet observé
	 * @param ju   	   : date julienne TU de la demande     
	 * @param longi_dg : longitude du lieu d'observation
	 * @param lati_dg  : latitude du lieu d'observation
	 * @param eco      : si true utilisation des paramètres variant lentement
	 * @param result   : Tableau de 7 doubles pour résultats renvoyés
	 * 		result[0] = jtran      date julienne TU du transit
	 *		result[1] = htran_dg   hauteur de l'objet au transit
	 *		result[2] = jlev       date julienne TU au lever
	 *		result[3] = azilev_dg  azimut au coucher
	 *		result[4] = jpon;      date julienne TU au coucher
	 *		result[5] = azipon_dg; azimut au coucher	
	 *		result[6] = h0_dg;     Hauteur de l'objet au coucher
	 * @return
	 */
	public static boolean getResult(Object o, double ju, double longi_dg, double lati_dg,
                                    boolean eco, double []result, int type)
	{
		double jtran, htran_dg; // date et hauteur transit
		boolean interOk = true; // Existence lever et coucher
		double jlev = -1, azilev_dg = 0.; // date et azimut au lever
		double jpon = -1, azipon_dg = 0.; // Date et azimut au coucher

		double aMeridien_dg;
		double asctra_dg, dectra_rd, dectra_dg;
		double asclev_dg, declev_rd, ascpon_dg, decpon_rd;
		double Hlev_rd = 0., Hpon_rd = 0., cosHlev = 1., cosHpon = 1.;
		double sinDecLev = 0., cosDecLev = 1., sinDecPon = 0., cosDecPon = 1.;

		double lati_rd = Math.toRadians(lati_dg);
		double []temp = {0}; // Pour les retours par pointeur
		// date UT de reference et hauteur au lever ou coucher de l'astre
		double jref = jrefEfho_dg(ju, o, temp, type); jref -=  longi_dg/360.; 
		double h0_dg = temp[0];		
		double sinh0 = Math.sin(Math.toRadians(h0_dg));
		// Ascension droite méridien local à jref
		aMeridien_dg = Astro.jul2agst_dg(jref, eco) + longi_dg;
		// Position de l'astre à la date de référence
		asctra_dg = Math.toDegrees(ju2equa_rd(ju, temp, eco, o));
		dectra_rd = temp[0]; dectra_dg = Math.toDegrees(dectra_rd);
		// Ecart d'ascension astre / au méridien jref		
		double deltaAsc_dg = Utils.realmod2(asctra_dg - aMeridien_dg, 360.);
		// Date estimée de passage au méridien de l'astre et hauteur correspondante
		jtran = jref + deltaAsc_dg/omegaT_dgj;
		// Position du méridien à la date transit estimée		
		htran_dg = 90 - Math.abs(lati_dg - dectra_dg);
		// Calcul lever et coucher
		double sinLa = Math.sin(lati_rd);
		double cosLa = Math.cos(lati_rd);
		double sinDec, cosDec, cosH;
		// Angle horaire estimée au lever et coucher
		sinDec = Math.sin(dectra_rd); cosDec = Math.cos(dectra_rd);
		cosH = (sinh0 - sinLa*sinDec)/(cosLa*cosDec);
		if (Math.abs(cosH) <= 1.)
		{
			double H_rd = Math.acos(cosH);
			double H_dg = Math.toDegrees(H_rd);
			jlev = jtran - H_dg/omegaT_dgj;
			jpon = jtran + H_dg/omegaT_dgj;
		}
		else interOk = false;	
		// 5 iterations maxi (1 à 3 suffisent en général) pour améliorer la précision
		double epstest = 0.1/86400; // à 0.1 s près
		double epstran, epslev = 1, epspon = 1;
		int iter;
		for(iter = 0; iter < 5; iter++) 
		{
			interOk = true;
			// Position de l'astre à la date transit estimée
			asctra_dg = Math.toDegrees(ju2equa_rd(jtran, temp, eco, o));
			dectra_rd = temp[0]; dectra_dg = Math.toDegrees(dectra_rd);
			// Ecart de position et correction de la date
			aMeridien_dg = Astro.jul2agst_dg(jtran, eco) + longi_dg;
			epstran = Utils.realmod2(asctra_dg - aMeridien_dg, 360.)/omegaT_dgj;
			jtran += epstran;
			htran_dg = 90 - Math.abs(lati_dg - dectra_dg);
			// Amélioration de la date lever estimée
			// Asc étoile a l'heure du lever
			asclev_dg = Math.toDegrees(ju2equa_rd(jlev, temp, eco, o));
			declev_rd = temp[0];	
			sinDecLev = Math.sin(declev_rd); cosDecLev = Math.cos(declev_rd);
			cosHlev = (sinh0 - sinLa*sinDecLev)/(cosLa*cosDecLev);
			if (Math.abs(cosHlev) <= 1.)
			{
				Hlev_rd = -Math.acos(cosHlev);
				// Asc méridien a l'heure du lever
				aMeridien_dg = Astro.jul2agst_dg(jlev, eco) + longi_dg;
				epslev = Utils.realmod2(asclev_dg + Math.toDegrees(Hlev_rd) - aMeridien_dg, 360.)/omegaT_dgj;
				jlev += epslev;
				// Amélioration de la date coucher estimée
				// Asc étoile a l'heure du coucher			
				ascpon_dg = Math.toDegrees(ju2equa_rd(jpon, temp, eco, o));
				decpon_rd = temp[0];	
				sinDecPon = Math.sin(decpon_rd); cosDecPon = Math.cos(decpon_rd);
				cosHpon = (sinh0 - sinLa*sinDecPon)/(cosLa*cosDecPon);
				if (Math.abs(cosHpon) <= 1.)
				{
					Hpon_rd = Math.acos(cosHpon);
					// Asc méridien a l'heure du coucher
					aMeridien_dg = Astro.jul2agst_dg(jpon, eco) + longi_dg;
					epspon = Utils.realmod2(ascpon_dg + Math.toDegrees(Hpon_rd) - aMeridien_dg, 360.)/omegaT_dgj;
					jpon += epspon;
				}
				else {jlev = -1; jpon = -1; interOk = false;}					
			}
			else {jlev = -1; jpon = -1; interOk = false;}			    			    	
			epstran = Math.abs(epstran);
			epslev = Math.abs(epslev);
			epspon = Math.abs(epspon);
			if (epstran > epstest) continue;
			if (!interOk) break; // jtran convergé et sous l'horizon
			if (epslev < epstest && 
					epspon < epstest) break; // tout a convergé
		}
		if(interOk) 
		{
			azilev_dg = Math.toDegrees(Math.atan2(Math.sin(Hlev_rd),
					sinLa*cosHlev - cosLa*sinDecLev/cosDecLev));
			azipon_dg = Math.toDegrees(Math.atan2(Math.sin(Hpon_rd),
					sinLa*cosHpon - cosLa*sinDecPon/cosDecPon));
		}
		result[0] = jtran;
		result[1] = htran_dg;
		result[2] = jlev;
		result[3] = azilev_dg;
		result[4] = jpon;
		result[5] = azipon_dg;
		result[6] = h0_dg;
		return interOk;
	}

	/**
	 * 
	 * @param result  : Les paramètres du transit calculés par boolean transit(...)
	 * @param sexaDec : Expression en sexagesimal si > 0, decimal sinon
	 * @param interOk : Si false données lever et coucher absentes
	 * @return        : Le string pret a etre affiche
	 */
	public static String getString(double []result, int sexaDec, boolean interOk)
	{
		String str = "";
		DecimalFormatSymbols dfs = new DecimalFormatSymbols(); dfs.setDecimalSeparator('.');
		DecimalFormat df4 = new DecimalFormat("0.0000", dfs);
		if (sexaDec > 0)
		{	
			if (interOk)
			{					
				str += "Date au lever      : " + Utils.jul2str1(result[2]) + " UT\n";
				str += "Azi/Sud du lever   : " + Utils.deg2dms2(result[3]) + "\n";
			}
			str += "Date transit       : " + Utils.jul2str1(result[0]) + " UT\n";
			str += "Hauteur transit    : " + Utils.deg2dms2(result[1]) + "\n";
			if (interOk)
			{					
				str += "Date au coucher    : " + Utils.jul2str1(result[4]) + " UT\n";
				str += "Azi/Sud du coucher : " + Utils.deg2dms2(result[5]) + "\n";
			}
		}
		else
		{				
			if (interOk)
			{									
				str += "Date au lever   : " + Utils.jul2str1(result[2]) + " UT\n";
				str += "Azi/Sud lever   : " + df4.format(result[3]) + "°\n";
			}
			str += "Date transit    : " + Utils.jul2str1(result[0]) + " UT\n";
			str += "Hauteur transit : " + df4.format(result[1]) + "°\n";
			if (interOk)
			{									
				str += "Date au coucher : " + Utils.jul2str1(result[4]) + " UT\n";
				str += "Azi/Sud coucher : " + df4.format(result[5]) + "°\n";
			}
		}
		if (!interOk)
		{
			if (result[1] < result[6]) str += "Toujours sous l'horizon !";
			else str += "Toujours au-dessus de l'horizon !"; 					
		}

		return str;
	}


	public static void main(String[] args) {
		double ju = 2431559.7;
		Planet p = new Planet(1); // Venus
		System.out.println("\nPosition de : " + p.nom);
		System.out.println(p.infos(ju, 0., 45., 1, true, true));
		ju = Utils.julNow();
		p = new Planet(2); // Soleil
		System.out.println("\nPosition de : " + p.nom);
		System.out.println(p.infos(ju, 0., 45., 1, true, true));
	}	
}
