package fr.llibre.ml2;

import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.Date;

public final class Utils {
/*------------ LISTE DES FONCTIONS STATIQUES ----------------------
Jour julien actuel a la date et heure courante
	double julNow()

Conversion degré décimal en dégré sexagésimal
	String deg2dms(double degdec) // 0 à 360
	String deg2dms2(double degdec) // -180 à +180

Conversion horaire décimal en horaire sexagésimal (partie décimale)
	String hdec2hms(double hhdec)

Conversion angle degré décimal en horaire sexagésimal
	String deg2hms(double ang)  // 0 à 24h
	String deg2hms2(double ang) //12h à +12h

Ecart TT-TU pour la date jour julien ju
	double jul2deltaT_s(double ju) // résultat en secondes
	double jul2deltaT(double ju) // résultat en jours

String date pour le jour julien jd
	String jul2str1 (double jd) // sans le jour de la semaine
	String jul2str2 (double jd) // avec le jour de la semaine

Remplacement des lettres accentuées dans un String
	String sansAccents(String s1) // par les lettres sans accent ou cédille, etc..

Modulos :
    double mod(a, b) // renvoi a modulo b, dans l'intervalle [0, b[.
    double mod2(a,b) // renvoi a modulo b, dans l'intervalle [-b/2, b/2[.

Lignes trigos inverses qui acceptent les arguments > à 1
	double acos(double a)
	double asin(double a)

-----------------------------------------------------------------*/
    /**
     * La classe Utils ne doit pas etre instanciee.
     */
    private Utils() {}

    public static double julNow()
    {
        Date nowDate = new Date();
        return (nowDate.getTime()/1000. - 946728000.)/86400. + 2451545.;
    }

    /*
     Tables des ecarts (TT - UT) tous les 5 ans pour les jours 1/1/1620 a 1/1/1970
     Pour interpoler, on considere que les dates charnieres sont equidistantes
     en nombre de jours entre les dates extremes.
     */
    private static double m1620 [][] = {
            {1620, 124},
            {1625, 102},
            {1630,  85},
            {1635,  72},
            {1640,  62},
            {1645,  54},
            {1650,  48},
            {1655,  43},
            {1660,  37},
            {1665,  32},
            {1670,  26},
            {1675,  21},
            {1680,  16},
            {1685,  12},
            {1690,  10},
            {1695,   9},
            {1700,  10},
            {1705,   9},
            {1710,  10},
            {1715,  10},
            {1720,  11},
            {1725,  11},
            {1730,  11},
            {1735,  12},
            {1740,  12},
            {1745,  13},
            {1750,  13},
            {1755,  14},
            {1760,  15},
            {1765,  16},
            {1770,  16},
            {1775,  17},
            {1780,  17},
            {1785,  17},
            {1790,  17},
            {1795,  16},
            {1800,  13.7},
            {1805,  12.6},
            {1810,  12.5},
            {1815,  12.5},
            {1820,  12.0},
            {1825,  10.2},
            {1830,   7.5},
            {1835,   5.8},
            {1840,  5.7},
            {1845,   6.3},
            {1850,   7.1},
            {1855,   7.6},
            {1860,   7.88},
            {1865,   6.02},
            {1870,   1.61},
            {1875,  -3.24},
            {1880,  -5.40},
            {1885,  -5.79},
            {1890,  -5.87},
            {1895,  -6.47},
            {1900,  -2.72},
            {1905,   3.86},
            {1910,  10.46},
            {1915,  17.20},
            {1920,  21.16},
            {1925,  23.62},
            {1930,  24.02},
            {1935,  23.93},
            {1940,  24.33},
            {1945,  26.77},
            {1950,  29.15},
            {1955,  31.07},
            {1960,  33.15},
            {1965,  35.73},
            {1970,  40.18}};

    /*
 Table des ecarts tous les 6 mois pour les dates 1/1/1972 0h au 1/7/2012 ....
 Les dates charnieres sont systematiquement les 1/1 a 12h et les 1/7 a 12h.
 http://www.stjarnhimlen.se/comp/time.html#deltat
     */
    private static double m1972 [][] = {
            {1972,  1,   42.23,  2441317.5},
            {1972,  7,   42.80,  2441499.5},
            {1973,  1,   43.37,  2441683.5},
            {1973,  7,   43.93,  2441864.5},
            {1974,  1,   44.49,  2442048.5},
            {1974,  7,   44.99,  2442229.5},
            {1975,  1,   45.48,  2442413.5},
            {1975,  7,   45.97,  2442594.5},
            {1976,  1,   46.46,  2442778.5},
            {1976,  7,   46.99,  2442960.5},
            {1977,  1,   47.52,  2443144.5},
            {1977,  7,   48.03,  2443325.5},
            {1978,  1,   48.53,  2443509.5},
            {1978,  7,   49.06,  2443690.5},
            {1979,  1,   49.59,  2443874.5},
            {1979,  7,   50.07,  2444055.5},
            {1980,  1,   50.54,  2444239.5},
            {1980,  7,   50.96,  2444421.5},
            {1981,  1,   51.38,  2444605.5},
            {1981,  7,   51.78,  2444786.5},
            {1982,  1,   52.17,  2444970.5},
            {1982,  7,   52.57,  2445151.5},
            {1983,  1,   52.96,  2445335.5},
            {1983,  7,   53.38,  2445516.5},
            {1984,  1,   53.79,  2445700.5},
            {1984,  7,   54.07,  2445882.5},
            {1985,  1,   54.34,  2446066.5},
            {1985,  7,   54.61,  2446247.5},
            {1986,  1,   54.87,  2446431.5},
            {1986,  7,   55.10,  2446612.5},
            {1987,  1,   55.32,  2446796.5},
            {1987,  7,   55.57,  2446977.5},
            {1988,  1,   55.82,  2447161.5},
            {1988,  7,   56.06,  2447343.5},
            {1989,  1,   56.30,  2447527.5},
            {1989,  7,   56.58,  2447708.5},
            {1990,  1,   56.86,  2447892.5},
            {1990,  7,   57.22,  2448073.5},
            {1991,  1,   57.57,  2448257.5},
            {1991,  7,   57.94,  2448438.5},
            {1992,  1,   58.31,  2448622.5},
            {1992,  7,   58.72,  2448804.5},
            {1993,  1,   59.12,  2448988.5},
            {1993,  7,   59.55,  2449169.5},
            {1994,  1,   59.98,  2449353.5},
            {1994,  7,   60.38,  2449534.5},
            {1995,  1,   60.78,  2449718.5},
            {1995,  7,   61.20,  2449899.5},
            {1996,  1,   61.63,  2450083.5},
            {1996,  7,   61.96,  2450265.5},
            {1997,  1,   62.29,  2450449.5},
            {1997,  7,   62.63,  2450630.5},
            {1998,  1,   62.97,  2450814.5},
            {1998,  7,   63.22,  2450995.5},
            {1999,  1,   63.47,  2451179.5},
            {1999,  7,   63.66,  2451360.5},
            {2000,  1,   63.82,  2451544.5},
            {2000,  7,   63.98,  2451726.5},
            {2001,  1,   64.09,  2451910.5},
            {2001,  7,   64.20,  2452091.5},
            {2002,  1,   64.30,  2452275.5},
            {2002,  7,   64.41,  2452456.5},
            {2003,  1,   64.47,  2452640.5},
            {2003,  7,   64.55,  2452821.5},
            {2004,  1,   64.57,  2453005.5},
            {2004,  7,   64.65,  2453187.5},
            {2005,  1,   64.68,  2453371.5},
            {2005,  7,   64.80,  2453552.5},
            {2006,  1,   64.85,  2453736.5},
            {2006,  7,   64.99,  2453917.5},
            {2007,  1,   65.15,  2454101.5},
            {2007,  7,   65.34,  2454282.5},
            {2008,  1,   65.45,  2454466.5},
            {2008,  7,   65.63,  2454648.5},
            {2009,  1,   65.78,  2454832.5},
            {2009,  7,   65.95,  2455013.5},
            {2010,  1,   66.07,  2455197.5},
            {2010,  7,   66.24,  2455378.5},
            {2011,  1,   66.32,  2455562.5},
            {2011,  7,   66.47,  2455743.5},
            {2012,  1,   66.60,  2455927.5},
            {2012,  7,   66.77,  2456109.5},
            {2013,  1,   66.95,  2456293.5},
            {2013,  7,   67.12,  2456474.5},
            {2014,  1,   67.30,  2456658.5},
            {2014,  7,   67.50,  2456839.5},
            {2015,  1,   67.67,  2457023.5},
            {2015,  7,   67.87,  2457204.5},
            {2016,  1,   68.10,  2457388.5},
            {2016,  7,   68.40,  2457570.5},
            {2017,  1,   68.5,   2457754.5}
    };

    /* Dates charnieres principales en jour julien */
    private static double JD948 = 2067314.5;
    private static double JD1620 = 2312752.5;
    private static double JD1970 = 2440587.5;
    private static double JD1972 = 2441317.5;
    private static double JD2017_1 = 2457754.5;
    private static double JD2050 = 2469807.5; // deltaT ~~ 85 s

    /*
        Interpolation de l'écart deltaT = TT-UT
        Entree : ju = jour julien
        Sortie : deltaT = TT - UT ecart en secondes entre le temps terrestre TT
                 (temps lineaire utilise pour le calcul des ephemerides) et le
                 temps universel UT irregulier, lie a la rotation terrestre.
     */
    public static double jul2deltaT_s(double ju)
    {
        double T;
        if (ju >= JD2017_1)
            return 68.5 + (ju - JD2017_1)*(85-68.5)/(JD2050-JD2017_1);
        if (ju >= JD1972) // Interpolation dans table semestrielle > 1972
        {
            int li1972 = m1972.length;;
            double delta6M = (JD2017_1 - JD1972)/(li1972-1);
            double deltaAA = ju - JD1972;
            int index = (int) (deltaAA/delta6M);
            if (ju < m1972[index][3]) index += 1;  // car duree delta6M variables
            double ratio = (ju - m1972[index][3])/(m1972[index+1][3] - m1972[index][3]);
            return m1972[index][2] + (m1972[index+1][2] - m1972[index][2])*ratio;
        }
        if (ju >= JD1970) // Interpolation entre les 2 tables
        {
            int end = m1620.length - 1;
            double ratio = (ju - JD1970)/(JD1972-JD1970);
            return m1620 [end][1] + ( m1972[0][2] - m1620[end][1] )*ratio;
        }
        if (ju >= JD1620) // Interpolation dans table 5-annuelle de 1620-1970
        {
            int li1620 = m1620.length;
            double delta5A = (JD1970 - JD1620)/(li1620-1);
            double deltaAA = ju - JD1620;
            int index = (int) (deltaAA/delta5A);
            double ratio = deltaAA/delta5A - index;
            return m1620[index][1] + ( m1620[index+1][1] - m1620[index][1])*ratio;
        }
        // Formules de Stephenson and Houlden (1986)
        T = (ju-2451545.0)/36525;
        if (ju >= JD948) return 50.6 + T*(67.5 + T*22.5) ;
        return 2715.6 + T*(573.36 + T*46.5);	}

    /*
    Interpolation de l'écart deltaT = TT-UT
    Entree : ju = jour julien
    Sortie : deltaT = TT - UT ecart en jours entre le temps terrestre TT
        (temps lineaire utilise pour le calcul des ephemerides) et le
        temps universel UT irregulier, lie a la rotation terrestre.
     */
    public static  double jul2deltaT(double ju)
    {
        return jul2deltaT_s(ju)/86400.;
    }


    /** Fonction convertissant les jours juliens en date TUC
     @param JJ    (double) date en jour julien
     @param jdec  Sortie jdec[0] : jour decimal (1.5 = le 1er e midi)
     @param m	  Sortie m[0] : mois (1 e 12)
     @param a	  Sortie a[0] : annee (attention 1 avant JC est note 0, etc..)
     @return      (String) le jour (Lundi, ...,Dimanche) de la semaine
     */
    public static String jul2dat (double JJ, double []jdec, int []m, int []a)
    {
        long   jj, jjm, jrs, mp1, ans, sic;
        String[]jours =
                {"Lundi", "Mardi", "Mercredi", "Jeudi","Vendredi", "Samedi", "Dimanche"};
		/* Decompte les siecles depuis le 1.5/3/-4801 en gregorien etendu
	       c'est-e-dire le le 7.5/4/-4801 en calendrier julien */
        jj = (long) (JJ + 0.5);
        sic = (100*jj + 3204500)/3652425;
		/* correction bissextiles julienne/gregorienne */
        jjm = jj + 1486;
        if (jj > 2299160.5) jjm += sic - sic/4; else jjm += 38;
		/* troncatures magiques prenant en compte l'origine au 1er mars */
        ans = (100*jjm - 12210)/36525;
        jrs = (36525*ans)/100;
        mp1 = ((jjm - jrs)*100)/3061;
        jdec[0] = jjm - jrs - ((306*mp1)/10) + JJ + 0.5 - jj;
        m[0] = (int) (mp1 - 1);    if (mp1 > 13) m[0] -= 12;
        a[0] = (int) (ans - 4715); if (m[0] > 2) a[0] -= 1;
        return jours[(int) (jj % 7)];
    }

    /** Fonction convertissant jour decimal en hms
     @param jdec  Entree jdec : jour decimal (1.5 = le 1er e midi)
     @param hh	  reférence sur l'heure
     @param mm	  référence sur les minutes
     @param ss	  référence sur les secondes
     */
    public static void jdec2hms(double jdec, int []hh, int []mm, double ss[])
    {
        int j = (int) jdec; // pour arrondi à la seconde près
        double r = (jdec - j)*24; // heures decimales
        int h = (int) r;
        r = (r - h)*60.; // minutes decimales
        int mn = (int) r;
        double s = ((r - mn)*60.); // secondes decimales
        hh[0] = h; mm[0] = mn; ss[0] = s;
    }



    /** Fonction convertissant la date TUC en string a la seconde pres
     @param jdec  Entree jdec[0] : jour decimal (1.5 = le 1er e midi)
     @param m	  Entree m[0] : mois (1 e 12)
     @param a	  Entree a[0] : annee (attention 1 avant JC est note 0, etc..)
     @return      (String) La date formatee J/M/A/ a XXhXXmnXXs
     */
    public static String dat2str(double []jdec, int []m, int []a)
    {
        int j = (int) (jdec[0] + 0.5/86400.); // pour arrondi à la seconde près
        double r = (jdec[0] - j)*24; // heures decimales
        int h = (int) r;
        r = (r - h)*60.; // minutes decimales
        int mn = (int) r;
        int s = (int) ((r - mn)*60.); // secondes arrondies
        if (s == 60){
            s = 0; mn = mn + 1;
            if (mn == 60){
                mn = 0; h = h+1;
            }
        }
        return " " + j + "/" + m[0] + "/" + a[0] + " à " + h + "h" +  mn + "m" +  s + "s";
    }

    /** Fonction convertissant les jours juliens en string date TUC
     * a la seconde pres, non précédé par le jour de la semaine
     @param JJ    (double) date en jour julien
     @return      (String) La date formatee J/M/A/ e XXhXXmnXXs
     */
    public static String jul2str1 (double JJ)
    {
        double []jdec = new double[]{0.};
        int [] a = new int []{0};
        int [] m = new int []{0};
        jul2dat (JJ, jdec, m, a);
        return dat2str(jdec, m, a) ;
    }

    /** Fonction convertissant les jours juliens en string date TUC
     * a la seconde pres, précédé par le jour de la semaine
     @param JJ    (double) date en jour julien
     @return      (String) La date formatee J/M/A/ e XXhXXmnXXs
     */
    public static String jul2str2 (double JJ)
    {
        double []jdec = new double[]{0.};
        int [] a = new int []{0};
        int [] m = new int []{0};
        String date = jul2dat (JJ, jdec, m, a);
        String result = date + dat2str(jdec, m, a) ;
        return result;
    }

    /*
     * realmod : Renvoi a modulo b compris entre 0 et b
     */
    public static double realmod(double a, double b)
    {
        return a - b* Math.floor(a/b);
    }

    /*
     * realmod2 : Renvoi a modulo b compris entre -b/2 et b/2
     */
    public static double realmod2(double a, double b)
    {
        return  a - b* Math.floor((a + b/2.)/b);
    }

    /**
     *   	Conversion date en jour julien
     * @param an
     * @param mois
     * @param jdec
     * @return
     */
    public static double dat2jul(int an, int mois, double jdec)
    {
        double JJ;
        long M, A;
		/* Changement origine des annees pour qu'elles soient > 0 afin que
	    les troncatures soient toutes dans le meme sens */
        A = an + 4800;
		/* Changement origine au 1er Mars, afin que la succession des jours
	    soit toujours la meme */
        M = mois; if (M <= 2) {A--; M += 12;}
		/* Correction Julienne/gregorienne */
        if (an <= 1582 && mois <= 10 && jdec < 15) JJ = -38.;
        else JJ = (A/400) - (A/100);
		/* Decompte avec troncature magique pour les 30/31 et correction des
	    changements d'origine */
        JJ += jdec + (36525*A/100) + (306*(M+1)/10) - 32167.5;
        return JJ;
    }



    /**
     *   	Conversion date en jour julien
     * @param an
     * @param mois
     * @param jour
     * @param heurs
     * @param mins
     * @param secs
     * @return
     */
    public static double dat2jul(int an, int mois, int jour, int heurs,
                                 int mins, double secs)
    {
        double j = ((secs/60. + mins)/60. + heurs)/24. + jour;
        double JJ;
        long M, A;
		/* Changement origine des annees pour qu'elles soient > 0 afin que
	    les troncatures soient toutes dans le meme sens */
        A = an + 4800;
		/* Changement origine au 1er Mars, afin que la succession des jours
	    soit toujours la meme */
        M = mois; if (M <= 2) {A--; M += 12;}
		/* Correction Julienne/gregorienne */
        if (an <= 1582 && mois <= 10 && j < 15) JJ = -38.;
        else JJ = (A/400) - (A/100);
		/* Decompte avec troncature magique pour les 30/31 et correction des
	    changements d'origine */
        JJ += j + (36525*A/100) + (306*(M+1)/10) - 32167.5;
        return JJ;
    }

    /*
      Conversion degré décimal en string ddd°mm'SS" avec 0° <= ddd < 360
     */
    public static String deg2dms(double degdec)
    {
        degdec = realmod(degdec, 360.);
        degdec += 1.e-10; // pour éviter les 60 secondes
        double deg = Math.floor(degdec);
        double minss = (degdec - deg)*60.;
        if (minss < 0.) minss = 0;
        double mm = Math.floor(minss + 1.e-13);
        double ss = (minss - mm)*60.;
        if (ss < 0.) ss = 0.;
        DecimalFormatSymbols sb = new DecimalFormatSymbols();
        sb.setDecimalSeparator('.');
        DecimalFormat df0 = new DecimalFormat("00", sb);
        String str = df0.format(deg) + "°" + df0.format(mm) + "'" + df0.format(ss) + "\"";
        return str;
    }

    /*
      Conversion degré décimal en string ddd°mm'SS" avec -180° < ddd <= 180
     */
    public static String deg2dms2(double degdec)
    {
        int signe;
        String str;
        degdec = realmod2(degdec, 360.);
        if (degdec < 0.){ signe = -1; degdec = -degdec;} else signe = 1;
        //		degdec += 0.5/(60.*60.); // arrondi fait par décimal format
        degdec += 1.e-10; // pour éviter les 60 secondes
        double deg = Math.floor(degdec);
        double minss = (degdec - deg)*60.;
        if (minss < 0.) minss = 0;
        double mm = Math.floor(minss + 1.E-13);
        double ss = (minss - mm)*60.;
        if (ss < 0.) ss = 0.;
        DecimalFormatSymbols sb = new DecimalFormatSymbols();
        sb.setDecimalSeparator('.');
        DecimalFormat df0 = new DecimalFormat("00", sb);
        //		DecimalFormat df3 = new DecimalFormat("00.000", sb);
        str = df0.format(deg) + "°" + df0.format(mm) + "'" + df0.format(ss) + "\"";
        if (signe < 0) str = "-" + str;
        return str;
    }

	/*
	  Conversion heure décimale en string HHhMMhSS avec 0 <= HH < 24
	 */

    private static String hdec2hms(double hhdec)
    {
        hhdec = realmod(hhdec, 24);
        hhdec += 1.e-10; // pour éviter les 60 secondes
        double hh = Math.floor(hhdec);
        double minss = (hhdec - hh)*60.;
        if (minss < 0.) minss = 0;
        double mm = Math.floor(minss);
        double ss = (minss - mm)*60.;
        if (ss < 0.) ss = 0.;
        DecimalFormatSymbols sb = new DecimalFormatSymbols();
        sb.setDecimalSeparator('.');
        DecimalFormat df0 = new DecimalFormat("00", sb);
        //		DecimalFormat df3 = new DecimalFormat("00.000", sb);
        String str = df0.format(hh) + "h" + df0.format(mm) + "m" + df0.format(ss) + "s";
        return str;
    }

    /*
      Conversion angle de degres en string HHhMMhSS.Ss avec 0 <= HH < 24
     */
    public static String deg2hms(double ang)
    {
        return hdec2hms(ang/15.);
    }

    /*
  Conversion angle de degres en string HHhMMhSS.Ss avec -12 < HH <= 12
     */
    public static String deg2hms2(double ang)
    {
        int signe;
        double hhdec = realmod2(ang/15., 24);
        if (hhdec >= 0.) signe = 1;
        else {signe = -1; hhdec = -hhdec; }
        String str = hdec2hms(hhdec);
        if (signe < 0) str = "-" + str;
        return str;
    }

    private static String lesA = "ÀÂÅÃÄÁ";
    private static String mesA = "AAAAAA";
    private static String lesE = "ÉÈÊË";
    private static String mesE = "EEEE";
    private static String lesI = "ÎÏÍ";
    private static String mesI = "III";
    private static String lesO = "ÔÓÖ";
    private static String mesO = "OOO";
    private static String lesU = "ÙÛÜ";
    private static String mesU = "UUU";
    private static String lesY = "Ÿ";
    private static String mesY = "Y";
    private static String lesC = "Ç";
    private static String mesC = "C";
    private static String lesN = "NÑ";
    private static String mesN = "NN";
    private static String lesS = "S";
    private static String mesS = "S";
    private static String strOri = lesA + lesE + lesI + lesO + lesU + lesY + lesC + lesN + lesS;
    private static String strRep = mesA + mesE + mesI + mesO + mesU + mesY + mesC + mesN + mesS;

    public static String sansAccents(String s1)
    {
        char [] c1 = s1.toCharArray();
        char c;
        boolean up;
        int k;
        for (int i = 0; i < c1.length; i++)
        {
            c = c1[i];
            up = Character.isUpperCase(c);
            if (!up) c = Character.toUpperCase(c);
            k = strOri.indexOf(c);
            if (k >= 0) c1[i] = strRep.charAt(k);
            if (!up) c1[i] = Character.toLowerCase(c1[i]);
        }
        return new String(c1);
    }

    /** Fonction renvoyant a modulo b compris entre 0 et b
     @param a    Le dividende (double)
     @param b    Le diviseur (double)
     @return     Le reste : a modulo b (double)
     */
    public static double mod(double a, double b)
    {
//		return (a - b*Math.floor(a/b)) ;
        double r = a%b;
        if (r < 0) r += b;
        return r;
    }

    /** Fonction renvoyant a modulo b compris entre -b/2 et b/2
     @param a    Le dividende (double)
     @param b    Le diviseur (double)
     @return     Le reste : a modulo b (double)
     */
    public static double mod2(double a, double b)
    {
        return (a - b* Math.floor((a+b/2.)/b)) ;
    }


    public static double acos(double a)
    {
        if (a >= 1.) return 0.;
        if (a <= -1) return Math.PI;
        return Math.acos(a);
    }

    public static double asin(double a)
    {
        if (a >= 1.) return Math.PI/2.;
        if (a <= -1) return -Math.PI/2.;
        return Math.asin(a);
    }

    public static void main(String[] args) {
        String un = "azÂserÓojkÈÊzéàùfg";
        System.out.println(un);
        System.out.println(sansAccents(un));
        System.out.println("deg2dms(400.25) = " + deg2dms(400.25));
        System.out.println("deg2dms(-400.25) = " + deg2dms(-400.25));
        System.out.println("deg2dms(560.25) = " + deg2dms(560.25));
        System.out.println("deg2dms(-560.25) = " + deg2dms(-560.25));
        System.out.println("");
        System.out.println("deg2dms2(400.25) = " + deg2dms2(400.25));
        System.out.println("deg2dms2(-400.25) = " + deg2dms2(-400.25));
        System.out.println("deg2dms2(560.25) = " + deg2dms2(560.25));
        System.out.println("deg2dms2(-560.25) = " + deg2dms2(-560.25));
        System.out.println("");
        System.out.println("hdec2hms(30.25) = " + hdec2hms(30.25));
        System.out.println("hdec2hms(-30.25) = " + hdec2hms(-30.25));
        System.out.println("hdec2hms(40.25) = " + hdec2hms(40.25));
        System.out.println("hdec2hms(-40.25) = " + hdec2hms(-40.25));
        System.out.println("");
        System.out.println("deg2hms(400.25) = " + deg2hms(400.25));
        System.out.println("deg2hms(-400.25) = " + deg2hms(-400.25));
        System.out.println("deg2hms(560.25) = " + deg2hms(560.25));
        System.out.println("deg2hms(-560.25) = " + deg2hms(-560.25));
        System.out.println("");
        System.out.println("deg2hms2(400.25) = " + deg2hms2(400.25));
        System.out.println("deg2hms2(-400.25) = " + deg2hms2(-400.25));
        System.out.println("deg2hms2(560.25) = " + deg2hms2(560.25));
        System.out.println("deg2hms2(-560.25) = " + deg2hms2(-560.25));
    }

}
